****************************************************************************************
* AUTOTRAN.PRG	: Transport Driver for GenScrnX (Dos/Win)
* Author		: Julian Ransom
* CIS Id		: 100415,2675
* Version		: 1.00
* Copyright		: None (Public Domain)
*
* Note:
*   This software is provided "as is" without
*   any warranty expressed or implied. In no event shall
*   the author (Julian Ransom) be liable for any damages.
*
* Description:
*	The Auto Transporter uses GenScrnX's transport hooks to pre- and post-process
*	screens which are being transported. It automatically transports all code snippets,
*	and allows you to keep certain objects from transporting (or even parts of an
*	object, such as the Picture clause). For objects which need user intervention,
*	it gives you full details of all the changes made to these objects.
*
* Usage:
*	To enable the AutoTran driver, place the following in your CONFIG.FP and .FPW:
* 		_TRNDRV1 = AUTOTRAN
* 		_TRNDRV1 = AUTOTRAN
*	and ensure that AutoTran.PRG is in your path.
*
* Notes on use:
*	When transportation begins, you will see a message window with the message
*	'Running Auto-Transporter...'. When this finishes, one of two things will
*	happen:
*		1.	The Fox Transport window comes up as usual.
*		2.	A text file appears on the screen. This gives you details of the
*			objects which the Fox Transport program is about to present you
*			with. For each object, the old and new values of each changed
*			field are shown.
*			Press <Esc> or <Ctrl-Enter> to continue with transportation. You
*			will then be presented with the Fox Transport window.
*
*	Once the Fox Transport window comes up, continue as normal. However, you
*	will probably find that having said 'OK' to see the list of objects to
*	transport, you go straight into your screen. This is because AutoTran
*	has handled these objects for you.
*	If you 'Cancel' in the Fox Transport window, all the changes that AutoTran
*	has made will be undone.
*
*	You may find that your Cleanup snippet has had a comment inserted at the top,
*		eg. *TIMESTAMP 511018346
*	Do not remove this comment - it is needed by the Auto-Transporter, and will
*	usually be removed during the next transport. If you remove it by mistake, 
*	the Auto-Transporter will adjust itself accordingly.
*	(If you look in your .SCX file, you will notice that the corresponding Header
*	 record will have temporarily been given a timestamp of 999999999. This is to
*	 prevent unnecessary re-transportation. The TIMESTAMP comment in the Cleanup
*	 snippet keeps a note of the original timestamp for the header.)
*
* Setup Snippet options:
*	*:NOTRANSPORT
*		Skip transportation for the whole screen.
*		Note that all code snippets will be transported anyway.
*
* Object Comment Snippet options:
*	*:NOTRANSPORT
*		Skip transportation for this object.
*		If it is a new object, it will _not_ appear on the other platform.
*		If it is an existing object, only code snippets will be transported.
*
*	*:NOTRANSPORT <Clause List - eg. PICTURE EXPR>
*		Transport this object, _except_ for the specified clause(s).
*		Note: code snippets are _always_ transported.
*
* Unusual Setup Snippet options:
*	*:NOTRANSPORT <Clause List - eg. PICTURE EXPR>
*	The Clause List is an optional list of SCX fields which will not be transported
*	for any objects.
*
*	$ specifies that if the _only_ difference between the old picture clause and
*	the new picture clause is the currency format, then the picture clause is not
*	to be transported.
*
* And finally:
*	Please feel free to send me your comments and suggestions, either
*	on CIS (see above) or the Internet (julian@ransom.demon.co.uk).
*
****************************************************************************************

PARAMETERS	cScxFile, nParam, lParam
PRIVATE		set_safe

m.set_safe	= set('SAFETY')
set safety OFF

if type('m.gcOldFile')='U'
	do TrnDrv1 with cScxFile
else
	do TrnDrv2 with cScxFile
endif

if m.set_safe='ON'
	set safety ON
endif


PROCEDURE	TrnDrv1
PARAMETERS	cSCXFile
PRIVATE		cPlatName, AllNoTrans, NoTransport, Different, i,;
			lShowDiff, lFPutsDone, fh, lAutoTransp,;
			cTextFile, cIDXName, cIDXFile, cDBFFile

* Pre-processing for TRANSPORT
			
m.lShowDiff	= .T.		&& Whether to display the differences or not
m.lFPutsDone= .F.		&& Whether the change file has been written to

m.lAutoTransp	= .F.
m.cPlatName	= iif(_DOS, 'DOS', 'WINDOWS')

use (m.cScxFile) in 0 alias OTHER		&& Other Platform
locate for platform=cPlatName
if found()
	m.lAutoTransp	= .T.
else
	use
endif

m.cTextFile	= sys(2023) + '\' + sys(3) + '.TXT'
m.cDBFFile	= sys(2023) + '\' + sys(3)
m.cIDXName	= sys(3)
m.cIDXFile	= sys(2023) + '\' + m.cIDXName

PUBLIC	gcOldFile
m.gcOldFile	= sys(2023) + '\' + sys(3)

if m.lAutoTransp
	wait window nowait 'Running Auto-Transporter...'
	if m.lShowDiff
		if file(m.cTextFile)
			erase (m.cTextFile)
		endif
		m.fh	= fcreate(m.cTextFile)
	endif

	copy to (m.gcOldFile + '.SCX')

	select OTHER
	index on uniqueid to (m.cIDXFile)
	set filter to platform#m.cPlatName

	use (m.cScxFile) again in 0 alias CURRENT	&& Current Platform
	select CURRENT
	set relation to uniqueid into OTHER

	* Check for *:NOTRANSPORT in the Cleanup code:
	select OTHER
	locate for objtype=1 and objcode=63
	DO CASE
	CASE OTHER.proccode='*TIMESTAMP'
		* The timestamp in the other platform was artificially updated to stop re-transportation
		* Put it back as it was:
		replace OTHER.timestamp	with val(substr(OTHER.proccode, 11, 10)),;
				OTHER.proccode	with substr(OTHER.proccode, 23)
	CASE OTHER.timestamp=999999999
		* The timestamp in the other platform was artificially updated to stop re-transportation
		* However, the note of the original timestamp has been removed.
		* Zero the timestamp.
		replace OTHER.timestamp with 0
	ENDCASE
	
	IF atc('*:NOTRANSPORT', OTHER.setupcode)>0
		m.AllNoTrans	= substr(OTHER.setupcode, atc('*:NOTRANSPORT', OTHER.setupcode) + 13) + chr(13)
		m.AllNoTrans	= upper(alltrim(left(m.AllNoTrans, at(chr(13), m.AllNoTrans) - 1))) + ' '
		if empty(m.AllNoTrans)
			m.AllNoTrans	= 'NONE '
		endif
	ELSE
		m.AllNoTrans	= ''
	ENDIF
	
	* Check changed entries
	select CURRENT
	scan for platform=m.cPlatName and !eof('OTHER') and Timestamp<OTHER.Timestamp
		IF atc('*:NOTRANSPORT', OTHER.comment)>0
			m.NoTransport	= substr(OTHER.comment, atc('*:NOTRANSPORT', OTHER.comment) + 13) + chr(13)
			m.NoTransport	= upper(alltrim(left(m.NoTransport, at(chr(13), m.NoTransport) - 1))) + ' '
			if empty(m.NoTransport)
				m.NoTransport	= 'NONE'
			endif
		ELSE
			m.NoTransport	= ''
		ENDIF
		m.NoTransport	= m.AllNoTrans + m.NoTransport
		
		_tally	= 0
		if !comment==OTHER.comment
			replace comment with OTHER.comment
		endif
		if !lotype==OTHER.lotype
			replace lotype with OTHER.lotype
		endif
		if !rangelo==OTHER.rangelo
			replace rangelo with OTHER.rangelo
		endif
		if !hitype==OTHER.hitype
			replace hitype with OTHER.hitype
		endif
		if !rangehi==OTHER.rangehi
			replace rangehi with OTHER.rangehi
		endif
		if !whentype==OTHER.whentype
			replace whentype with OTHER.whentype
		endif
		if !when==OTHER.when
			replace when with OTHER.when
		endif
		if !validtype==OTHER.validtype
			replace validtype with OTHER.validtype
		endif
		if !valid==OTHER.valid
			replace valid with OTHER.valid
		endif
		if !errortype==OTHER.errortype
			replace errortype with OTHER.errortype
		endif
		if !error==OTHER.error
			replace error with OTHER.error
		endif
		if !messtype==OTHER.messtype
			replace messtype with OTHER.messtype
		endif
		if !message==OTHER.message
			replace message with OTHER.message
		endif
		if !showtype==OTHER.showtype
			replace showtype with OTHER.showtype
		endif
		if !show==OTHER.show
			replace show with OTHER.show
		endif
		if !activtype==OTHER.activtype
			replace activtype with OTHER.activtype
		endif
		if !activate==OTHER.activate
			replace activate with OTHER.activate
		endif
		if !deacttype==OTHER.deacttype
			replace deacttype with OTHER.deacttype
		endif
		if !deactivate==OTHER.deactivate
			replace deactivate with OTHER.deactivate
		endif
		if !proctype==OTHER.proctype
			replace proctype with OTHER.proctype
		endif
		if !proccode==OTHER.proccode
			replace proccode with OTHER.proccode
		endif
		if !setupcode==OTHER.setupcode
			replace setupcode with OTHER.setupcode
		endif
		if !setuptype==OTHER.setuptype
			replace setuptype with OTHER.setuptype
		endif
		if !initialval==OTHER.initialval
			replace initialval with OTHER.initialval
		endif
		if '$' $ m.NoTransport and !'PICTURE ' $ m.NoTransport and !picture==OTHER.picture ;
				and strtran(picture, '$', '') == strtran(other.picture, '$', '')
			* The PICTURE field is different, but the only difference
			* is the currency operator.
			* "NOTRANSPORT $" has been specified, so ignore this difference.
			_tally	= _tally + 1
			m.NoTransport	= m.NoTransport + 'PICTURE '
		endif
		if _tally>0 or m.lShowDiff
			* Check and show the differences:
			m.different	= ''
			for i=1 to fcount()
				if !inlist(field(m.i), 'PLATFORM', 'TIMESTAMP', 'HEIGHT', 'WIDTH', 'HPOS', 'VPOS', 'PEN', 'FILL', 'FONT', 'GRID', 'SCHEME', 'MODE', 'SPACING', 'COLORPAIR', 'CURPOS', 'SHADOW', 'MINIMIZE', 'ORDER', 'TAG') ;
						and !trim(field(m.i))+' ' $ m.NoTransport ;
						and !eval(field(m.i))==eval('OTHER.' + field(m.i))
					m.different	= m.different + field(m.i) + chr(13) + chr(10)
				endif
			next
			do case
			case empty(m.different) or m.NoTransport='NONE'
				replace timestamp with OTHER.timestamp
			case m.lShowDiff
				m.lFPutsDone	= .T.
				if objtype=15 and objcode=0
					= fputs(m.fh, 'SAY Object')
				else
					= fputs(m.fh, iif(empty(name), expr, name))
				endif
				_mline	= 0
				for i=1 to memlines(m.different)
					m.cField	= mline(m.different, 1, _mline)
					if !empty(m.cField)
						= fputs(m.fh, '  ' + m.cField)
						= fwrite(m.fh, '    Old' + chr(9))
						do case
						case inlist(type(m.cField), 'C', 'M')
							= fputs(m.fh, space(6) + eval(m.cField))
						case type(m.cField)='N'
							= fputs(m.fh, space(6) + str(eval(m.cField)))
						case type(m.cField)='D'
							= fputs(m.fh, space(6) + dtoc(eval(m.cField)))
						case type(m.cField)='L'
							= fputs(m.fh, space(6) + iif(eval(m.cField), '.T.', '.F.'))
						endcase
						= fwrite(m.fh, '    New' + chr(9))
						do case
						case inlist(type(m.cField), 'C', 'M')
							= fputs(m.fh, space(6) + eval('OTHER.' + m.cField))
						case type(m.cField)='N'
							= fputs(m.fh, space(6) + str(eval('OTHER.' + m.cField)))
						case type(m.cField)='D'
							= fputs(m.fh, space(6) + dtoc(eval('OTHER.' + m.cField)))
						case type(m.cField)='L'
							= fputs(m.fh, space(6) + iif(eval('OTHER.' + m.cField), '.T.', '.F.'))
						endcase
						= fputs(m.fh, '')
					endif
				next
			endcase
		endif
	endscan
	
	if m.lShowDiff
		= fclose(m.fh)
		if m.lFPutsDone
			wait clear
			modify file (m.cTextFile)
			wait window nowait 'Running Auto-Transporter...'
		endif
		if file(m.cTextFile)
			erase (m.cTextFile)
		endif
	endif
	
	* Check NEW objects:
	set relation to
	set order to (m.cIDXName)
	set filter to platform=m.cPlatName
	
	select OTHER
	set order to
	set relation to uniqueid into CURRENT
	
	* Log the timestamps (which act as the IDs) of all Grouped Boxes
	m.cGroups	= ''
	scan for objtype=10 and objcode=1
		m.cGroups	= m.cGroups + str(timestamp, 10) + ' '
	endscan
	
	copy for eof('CURRENT') and (objtype=7 or objtype=6;
			or (objtype#1 and '*:NOTRANSPORT'$comment and len(alltrim(mline(comment, atcline('*:NOTRANSPORT', comment))))=13);
			or str(timestamp, 10)$m.cGroups);
		to (m.cDBFFile)
	set relation to
	
	if _tally>0
		select 0
		use (m.cDBFFile)
		replace all platform	with m.cPlatName,;
					comment		with 'TRNDRV1'
		use
		
		select CURRENT
		set filter to
		set order to
		* Make a note of the Header's old timestamp, and make it the most recent object
		* This will be undone next time this program is run
		replace for platform=m.cPlatName and objtype=1 and objcode=63 ;
			proccode	with '*TIMESTAMP' + str(timestamp, 10) + chr(13) + chr(10) + proccode,;
			timestamp	with 999999999
		append from (m.cDBFFile)
	endif
	
	if file(m.cDBFFile)
		erase (m.cDBFFile + '.DBF')
		erase (m.cDBFFile + '.FPT')
	endif
	
	use in CURRENT
	use in OTHER
	
	erase (m.cIDXFile + '.IDX')
	
	wait clear
endif



PROCEDURE	TrnDrv2
PARAMETERS	cScxFile

* Post-processing for TRANSPORT

if m.gstatus=1
	
	use (m.cScxFile) in 0 alias CURRENT exclusive
	
	select CURRENT
	delete for comment=='TRNDRV1'
	
	if _tally>0
		pack
	endif
	
	use
	
else
	if file(gcOldFile + '.SCX')
		use (gcOldFile + '.SCX')
		copy to (m.cScxFile)
		wait window nowait 'Changes have been abandoned.'
		use
	endif
endif

if file(gcOldFile + '.SCX')
	erase (gcOldFile + '.SCX')
	erase (gcOldFile + '.SCT')
endif

RELEASE gcOldFile
